@@ -10,7 +10,7 @@
  * Copyright (C) 2003
  *  Antti Tapaninen
  *
- * $Id: hotplug_macosx.c 5343 2010-10-22 11:52:22Z rousseau $
+ * $Id: hotplug_macosx.c 5508 2011-01-14 08:44:52Z rousseau $
  */
 
 /**
@@ -112,7 +112,6 @@ static void HPDeviceDisappeared(void *refCon, io_iterator_t iterator)
  */
 static HPDriverVector HPDriversGetFromDirectory(const char *driverBundlePath)
 {
-	int i;
 #ifdef DEBUG_HOTPLUG
 	Log2(PCSC_LOG_DEBUG, "Entering HPDriversGetFromDirectory: %s",
 		driverBundlePath);
@@ -145,6 +144,7 @@ static HPDriverVector HPDriversGetFromDirectory(const char *driverBundlePath)
 	CFRelease(pluginUrl);
 
 	size_t bundleArraySize = CFArrayGetCount(bundleArray);
+	size_t i;
 
 	/* get the number of readers (including aliases) */
 	for (i = 0; i < bundleArraySize; i++)
@@ -759,12 +759,10 @@ LONG HPSearchHotPluggables(void)
 		{
 			char deviceName[MAX_DEVICENAME];
 
-			/* the format should be "usb:%04x/%04x:libusb:%s" but we do not
-			 * know the libusb string. So it is not possible to differentiate
-			 * two identical readers :-( */
-			snprintf(deviceName, sizeof(deviceName), "usb:%04x/%04x",
-				(unsigned int)a->m_driver->m_vendorId,
-				(unsigned int)a->m_driver->m_productId);
+			/* the format should be "usb:%04x/%04x" but Apple uses the
+			 * friendly name instead */
+			snprintf(deviceName, sizeof(deviceName),
+				"%s", a->m_driver->m_friendlyName);
 			deviceName[sizeof(deviceName)-1] = '\0';
 
 			RFAddReader(a->m_driver->m_friendlyName,
