@@ -4,7 +4,7 @@
  * Copyright (C) 2006-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 2377 2007-02-05 13:13:56Z rousseau $
+ * $Id: utils.c 5674 2011-03-26 19:53:47Z rousseau $
  */
 
 /**
@@ -31,18 +31,19 @@
 
 pid_t GetDaemonPid(void)
 {
-	FILE *f;
+	int fd;
 	pid_t pid;
 
 	/* pids are only 15 bits but 4294967296
 	 * (32 bits in case of a new system use it) is on 10 bytes
 	 */
-	if ((f = fopen(PCSCLITE_RUN_PID, "rb")) != NULL)
+	fd = open(PCSCLITE_RUN_PID, O_RDONLY);
+	if (fd >= 0)
 	{
 		char pid_ascii[PID_ASCII_SIZE];
 
-		(void)fgets(pid_ascii, PID_ASCII_SIZE, f);
-		(void)fclose(f);
+		(void)read(fd, pid_ascii, PID_ASCII_SIZE);
+		(void)close(fd);
 
 		pid = atoi(pid_ascii);
 	}
@@ -91,7 +92,7 @@ int CheckForOpenCT(void)
 
 	if (0 == stat(OPENCT_FILE, &buf))
 	{
-		Log1(PCSC_LOG_CRITICAL, "Remove OpenCT and try again");
+		Log1(PCSC_LOG_CRITICAL, "File " OPENCT_FILE " found. Remove OpenCT and try again");
 		return 1;
 	}
 
